/*!
 * Nodeunit
 * Copyright (c) 2010 Caolan McMahon
 * Modifications in devtools Copyright (c) 2011 Bill Barnhill
 * MIT Licensed
 */

/*
 * Copied from https://raw.github.com/caolan/nodeunit/master/lib/track.js
 * version: 0.5.1  (commit 9d511ff0c3f1e1809a58)
 * Modified by Bill Barnhill to watch lib dir and do continuous reporting
 */

/**
 * Module dependencies
 */

var nodeunit = require('nodeunit'),
  utils = nodeunit.utils,
  fs = require('fs'),
  sys = require('sys'),
  track = require('../track'),
  path = require('path'),
  stalker = require('stalker');

AssertionError = require('../assert').AssertionError; 

/*
 * Notes:
 * 1. For now you'll need to ctrl-c out. I need
 *    to add support for start/stop
 */

/**
 * Reporter info string
 */

exports.info = "Default tests reporter";


/**
 * Run all tests within each module, reporting the results to the command-line.
 * Watch 'test' and 'lib' directories (all the way down) for changed files.
 * When change happeneds schedule a test run in 5 seconds (need to put that in 
 * opts), if tests not already scheduled.
 *
 * @param {Array} files
 * @api public
 */

exports.run = function (files, options) {
  if (!options) {
    // load default options
    var content = fs.readFileSync(
				  __dirname + '/../nodeunit.json', 'utf8'
				  );
    options = JSON.parse(content);
  }

  process.on('uncaughtException', function (err) {
      console.log('Caught exception: ' + err);
      console.log('Now exiting...');
      process.reallyExit(1);
    });
 
  console.log("Use Ctrl-C to exit for now, start/support is coming...");

  var changeHistory = [];
  var changeHistorySize = 100;
  var scheduled = false;

  stalker.watch('lib', onChange);
  stalker.watch('test', onChange);

  function onChange(err, file) {
      console.log('Tests will run soon due to changed file : '+file);
      var change = {
	'file' : file,
	'modified' : new Date()
      };
      if (changeHistory.push(change) > changeHistorySize) {
	while (changeHistory.length > changeHistorySize) {
	  changeHistory.shift();
	}
      }
      if (file in require.cache)
	delete require.cache[file]; // this forces reload, but is fragile
      
      changeHistory.dirty = true;
      if (!scheduled) {
	scheduled = true;
	setTimeout(function() {
	    doPass(files, options);
	    scheduled = false;
	  }, 5000);
      }
      //doPass(files, options);
    } 
}

function doPass(files, options) {

  var error = function (str) {
    return options.error_prefix + str + options.error_suffix;
  };
  var ok    = function (str) {
    return options.ok_prefix + str + options.ok_suffix;
  };
  var bold  = function (str) {
    return options.bold_prefix + str + options.bold_suffix;
  };
  var assertion_message = function (str) {
    return options.assertion_prefix + str + options.assertion_suffix;
  };

  var start = new Date().getTime();
  var paths = files.map(function (p) {
      return path.join(process.cwd(), p);
    });
  var tracker = track.createTracker(function (tracker) {
      if (tracker.unfinished()) {
	sys.puts('');
	sys.puts(error(bold(
			    'FAILURES: Undone tests (or their setups/teardowns): '
			    )));
	var names = tracker.names();
	for (var i = 0; i < names.length; i += 1) {
	  sys.puts('- ' + names[i]);
	}
	sys.puts('Exiting, due to broken tests.');
	sys.puts('To fix this, make sure all tests call test.done()');
	process.reallyExit(tracker.unfinished());
      }
    });

  nodeunit.runFiles(paths, {
    moduleStart: function (name) {
	sys.puts('\n' + bold(name));
      },
	testDone: function (name, assertions) {
	tracker.remove(name);

	if (!assertions.failures()) {
	  sys.puts('✔ ' + name);
	}
	else {
	  sys.puts(error('✖ ' + name) + '\n');
	  assertions.forEach(function (a) {
	      if (a.failed()) {
		a = utils.betterErrors(a);
		// nodeunit's AssertionError is not ours, so changed next line
		if (('message' in a) && a.message) {
		  sys.puts(
			   'Assertion Message: ' +
			   assertion_message(a.message)
			   );
		}
		sys.puts(a.error.stack + '\n');
	      }
	    });
	}
      },
	done: function (assertions) {
	var end = new Date().getTime();
	var duration = end - start;
	if (assertions.failures()) {
	  sys.puts(
		   '\n' + bold(error('FAILURES: ')) + assertions.failures() +
		   '/' + assertions.length + ' assertions failed (' +
		   assertions.duration + 'ms)'
		   );
	}
	else {
	  sys.puts(
		   '\n' + bold(ok('OK: ')) + assertions.length +
		   ' assertions (' + assertions.duration + 'ms)'
		   );
	}
      },
	testStart: function(name) {
	tracker.put(name);
      }
    });
};
